(* Exam Test Framework  *)
(* ©2009 Marwan Burelle *)

let random_printable_char =
  let ca = Char.code 'a' in
    fun _ -> Char.chr ((Random.int 26) + ca)

let rec makeList f l = function
    0 -> l
  | n when n>0 -> makeList f (f()::l) (n-1)
  | _ -> assert false

let makePair m =
  (Random.int m, Random.int m)

let pair_to_string (x,y) =
  Printf.sprintf "(%02i,%02i)" x y

let printListpref to_s l =
  begin
    Format.printf "l = @[<b 2>[@;";
    List.iter (fun x -> Format.printf "%s;@;" (to_s x)) l;
    Format.printf "]@]@.";
  end

let printList to_s l =
  begin
    Format.printf "[@;";
    List.iter (fun x -> Format.printf "%s;@;" (to_s x)) l;
    Format.printf "]";
  end

class question n =
object (s)
  val q = Queue.create ()
  method getQ = q
  val mutable plist = []
  method add_plist p = plist <- p::plist
  method get_plist = plist
  method quest = "Question " ^ (string_of_int n) ^ ":\n"
  method paramline =
    (Printf.sprintf "./question%02i " n)
    ^(List.fold_left (fun s p -> s ^ p) "" plist)
  method usage = s#quest ^ s#paramline
  method beginParse =
    Format.set_margin 60;
    let anonfun s = Queue.add s q in
      Arg.parse (Arg.align []) anonfun s#usage
  method die:unit = Arg.usage (Arg.align []) (s#usage); exit 1
end

class virtual qSeed =
object (s)
  method virtual getQ : string Queue.t
  method virtual usage : string
  method virtual die : unit
  method init =
    try
      Random.init (int_of_string (Queue.take s#getQ))
    with
	_ -> s#die
end

class virtual qSize =
object (s)
  method virtual getQ : string Queue.t
  method virtual usage : string
  method virtual die : unit
  val mutable size = None
  method getSize =
    match size with
	None ->
	  begin
	    try
	      size <- Some (int_of_string (Queue.take s#getQ));
	      s#getSize
	    with
		_ -> s#die; 0
	  end
      | Some n -> n
end

class qInt n m =
object (s)
  inherit question n
  inherit qSeed
  method getN = Random.int m
  initializer
    s#add_plist "graine "
end

class qFloat n m =
object (s)
  inherit question n
  inherit qSeed
  method getN = Random.float m
  initializer
    s#add_plist "graine "
end

class qSeedSizeInt n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  method getN = Random.int m
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end

class qIntList n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  method makeList =
    makeList (fun () -> Random.int m) [] s#getSize
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end

class qPairIntList n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  val mutable max = m
  method set_max m = max <- m
  method makeList =
    makeList (fun () -> makePair max) [] s#getSize
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end

class qSizeString n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  method makeString =
    let s = String.create m in
      for i = 0 to m-1 do
	s.[i] <- random_printable_char ()
      done;
      s
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end
